package store

import (
	"crypto/rand"
	"fmt"
	"math/big"
	"os"
)

// SecretNew generates a new user secret.
func (s *Store) SecretNew() string {
	secret := make([]byte, 64)
	for i := 0; i < 64; i++ {
		if n, err := rand.Int(rand.Reader, big.NewInt(int64(len(letters)))); err != nil {
			s.fatalClose(fmt.Sprintf("Error generating secret, %s", err))
		} else {
			secret[i] = letters[n.Int64()]
		}
	}
	return string(secret)
}

// SecretLookup looks up a user from a secret.
func (s *Store) SecretLookup(secret string) User {
	if !secretRegex.MatchString(secret) || !s.file(s.SecretPath(secret)) {
		return User{}
	}
	if !s.Compat {
		return s.user(s.SecretPath(secret) + "/" + infoJson)
	} else {
		if path, err := os.ReadFile(s.SecretPath(secret)); err != nil {
			s.fatalClose(fmt.Sprintf("Error reading association file of secret %s, %s", secret, err))
		} else {
			return s.user(string(path) + "/" + infoJson)
		}
	}
	return User{}
}

// SecretAssociate associates a secret with a user.
func (s *Store) SecretAssociate(secret, flake string) {
	if s.file(s.SecretPath(secret)) {
		return
	}
	s.link("../users/"+flake, s.SecretPath(secret))
}

// SecretDisassociate disassociates a secret.
func (s *Store) SecretDisassociate(secret string) {
	if !s.file(s.SecretPath(secret)) {
		return
	}
	if err := os.Remove(s.SecretPath(secret)); err != nil {
		s.fatalClose(fmt.Sprintf("Error disassociating secret %s, %s", secret, err))
	}
}
